from datetime import datetime
from typing import List

from mercapi.models.base import ResponseModel, Extractors, ResponseProperty
from mercapi.models.common import ItemCategory
from mercapi.models.item.data import (
    ItemCondition,
    ShippingFromArea,
    ShippingMethod,
    ShippingDuration,
    ShippingClass,
    ShippingPayer,
    Color,
    Seller,
    Comment,
)


# actually a mapping for 'data' object in item response
#
# consider nesting all the properties in data submodel
# if the original response gets more verbose
class Item(ResponseModel):
    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty("status", "status", Extractors.get("status")),
        ResponseProperty("name", "name", Extractors.get("name")),
        ResponseProperty("price", "price", Extractors.get("price")),
    ]
    _optional_properties = [
        ResponseProperty("seller", "seller", Extractors.get_as_model("seller", Seller)),
        ResponseProperty("description", "description", Extractors.get("description")),
        ResponseProperty("photos", "photos", Extractors.get("photos")),
        ResponseProperty("photo_paths", "photo_paths", Extractors.get("photo_paths")),
        ResponseProperty("thumbnails", "thumbnails", Extractors.get("thumbnails")),
        ResponseProperty(
            "item_category",
            "item_category",
            Extractors.get_as_model("item_category", ItemCategory),
        ),
        ResponseProperty(
            "item_condition",
            "item_condition",
            Extractors.get_as_model("item_condition", ItemCondition),
        ),
        ResponseProperty(
            "colors",
            "colors",
            Extractors.get_list_with("colors", lambda x: Color.from_dict(x)),
        ),
        ResponseProperty(
            "shipping_payer",
            "shipping_payer",
            Extractors.get_as_model("shipping_payer", ShippingPayer),
        ),
        ResponseProperty(
            "shipping_method",
            "shipping_method",
            Extractors.get_as_model("shipping_method", ShippingMethod),
        ),
        ResponseProperty(
            "shipping_from_area",
            "shipping_from_area",
            Extractors.get_as_model("shipping_from_area", ShippingFromArea),
        ),
        ResponseProperty(
            "shipping_duration",
            "shipping_duration",
            Extractors.get_as_model("shipping_duration", ShippingDuration),
        ),
        ResponseProperty(
            "shipping_class",
            "shipping_class",
            Extractors.get_as_model("shipping_class", ShippingClass),
        ),
        ResponseProperty("num_likes", "num_likes", Extractors.get("num_likes")),
        ResponseProperty(
            "num_comments", "num_comments", Extractors.get("num_comments")
        ),
        ResponseProperty(
            "comments",
            "comments",
            Extractors.get_list_with("comments", lambda x: Comment.from_dict(x)),
        ),
        ResponseProperty("updated", "updated", Extractors.get_datetime("updated")),
        ResponseProperty("created", "created", Extractors.get_datetime("created")),
        ResponseProperty("pager_id", "pager_id", Extractors.get("pager_id")),
        ResponseProperty("liked", "liked", Extractors.get("liked")),
        ResponseProperty("checksum", "checksum", Extractors.get("checksum")),
        ResponseProperty(
            "is_dynamic_shipping_fee",
            "is_dynamic_shipping_fee",
            Extractors.get("is_dynamic_shipping_fee"),
        ),
        # unknown schema:
        ResponseProperty(
            "application_attributes",
            "application_attributes",
            Extractors.get("application_attributes"),
        ),
        ResponseProperty(
            "is_shop_item", "is_shop_item", Extractors.get("is_shop_item")
        ),
        ResponseProperty(
            "is_anonymous_shipping",
            "is_anonymous_shipping",
            Extractors.get("is_anonymous_shipping"),
        ),
        ResponseProperty(
            "is_web_visible", "is_web_visible", Extractors.get("is_web_visible")
        ),
        ResponseProperty(
            "is_offerable", "is_offerable", Extractors.get("is_offerable")
        ),
        ResponseProperty(
            "is_organizational_user",
            "is_organizational_user",
            Extractors.get("is_organizational_user"),
        ),
        ResponseProperty(
            "organizational_user_status",
            "organizational_user_status",
            Extractors.get("organizational_user_status"),
        ),
        ResponseProperty(
            "is_stock_item", "is_stock_item", Extractors.get("is_stock_item")
        ),
        ResponseProperty(
            "is_cancelable", "is_cancelable", Extractors.get("is_cancelable")
        ),
        ResponseProperty(
            "shipped_by_worker",
            "shipped_by_worker",
            Extractors.get("shipped_by_worker"),
        ),
        # unknown list, ignore: additional_services
        ResponseProperty(
            "has_additional_service",
            "has_additional_service",
            Extractors.get("has_additional_service"),
        ),
        ResponseProperty(
            "has_like_list", "has_like_list", Extractors.get("has_like_list")
        ),
        ResponseProperty(
            "is_offerable_v2", "is_offerable_v2", Extractors.get("is_offerable_v2")
        ),
    ]

    def __init__(
        self,
        id_: str,
        seller: Seller,
        status: str,
        name: str,
        price: int,
        description: str,
        photos: List[str],
        photo_paths: List[str],
        thumbnails: List[str],
        item_category: ItemCategory,
        item_condition: ItemCondition,
        colors: List[Color],
        shipping_payer: ShippingPayer,
        shipping_method: ShippingMethod,
        shipping_from_area: ShippingFromArea,
        shipping_duration: ShippingDuration,
        shipping_class: ShippingClass,
        num_likes: int,
        num_comments: int,
        comments: List[Comment],
        updated: datetime,
        created: datetime,
        pager_id: int,
        liked: bool,
        checksum: str,
        is_dynamic_shipping_fee: bool,
        application_attributes: dict,
        is_shop_item: str,
        is_anonymous_shipping: bool,
        is_web_visible: bool,
        is_offerable: bool,
        is_organizational_user: bool,
        organizational_user_status: str,
        is_stock_item: bool,
        is_cancelable: bool,
        shipped_by_worker: bool,
        has_additional_service: bool,
        has_like_list: bool,
        is_offerable_v2: bool,
    ):
        super().__init__()
        self.id_ = id_
        self.seller = seller
        self.status = status
        self.name = name
        self.price = price
        self.description = description
        self.photos = photos
        self.photo_paths = photo_paths
        self.thumbnails = thumbnails
        self.item_category = item_category
        self.item_condition = item_condition
        self.colors = colors
        self.shipping_payer = shipping_payer
        self.shipping_method = shipping_method
        self.shipping_from_area = shipping_from_area
        self.shipping_duration = shipping_duration
        self.shipping_class = shipping_class
        self.num_likes = num_likes
        self.num_comments = num_comments
        self.comments = comments
        self.updated = updated
        self.created = created
        self.pager_id = pager_id
        self.liked = liked
        self.checksum = checksum
        self.is_dynamic_shipping_fee = is_dynamic_shipping_fee
        self.application_attributes = application_attributes
        self.is_shop_item = is_shop_item
        self.is_anonymous_shipping = is_anonymous_shipping
        self.is_web_visible = is_web_visible
        self.is_offerable = is_offerable
        self.organizational_user_status = organizational_user_status
        self.is_organizational_user = is_organizational_user
        self.is_stock_item = is_stock_item
        self.is_cancelable = is_cancelable
        self.shipped_by_worker = shipped_by_worker
        self.has_additional_service = has_additional_service
        self.has_like_list = has_like_list
        self.is_offerable_v2 = is_offerable_v2



import uuid
from dataclasses import dataclass, field
from enum import Enum
from typing import List

from mercapi.requests import RequestData


class SearchRequestData(RequestData):
    class ShippingMethod(Enum):
        SHIPPING_METHOD_ANONYMOUS = 1
        SHIPPING_METHOD_JAPAN_POST = 2
        SHIPPING_METHOD_NO_OPTION = 3

    class Status(Enum):
        STATUS_ON_SALE = 1
        STATUS_SOLD_OUT = 2
        # STATUS_TRADING = 3

    @dataclass
    class SearchConditions:
        query: str
        categories: List[int] = field(default_factory=list)
        brands: List[int] = field(default_factory=list)
        sizes: List[int] = field(default_factory=list)
        price_min: int = 0
        price_max: int = 0
        item_conditions: List[int] = field(default_factory=list)
        shipping_payer: List[int] = field(default_factory=list)
        colors: List[int] = field(default_factory=list)
        shipping_methods: List["SearchRequestData.ShippingMethod"] = field(
            default_factory=list
        )
        status: List["SearchRequestData.Status"] = field(default_factory=list)

    def __init__(self, search_conditions: SearchConditions):
        super().__init__()
        (
            query,
            categories,
            brands,
            sizes,
            price_min,
            price_max,
            item_conditions,
            shipping_payer,
            colors,
        ) = map(
            search_conditions.__dict__.get,
            (
                "query",
                "categories",
                "brands",
                "sizes",
                "price_min",
                "price_max",
                "item_categories",
                "shipping_payer",
                "colors",
            ),
        )
        shipping_methods = [i.name for i in search_conditions.shipping_methods]
        status = [i.name for i in search_conditions.status]
        if "STATUS_SOLD_OUT" in status:
            status.extend("STATUS_TRADING")

        self.data = {
            "userId": "",
            "pageSize": 120,
            "pageToken": "",
            "searchSessionId": uuid.uuid4().hex,
            "indexRouting": "INDEX_ROUTING_UNSPECIFIED",
            "thumbnailTypes": [],
            "searchCondition": {
                "keyword": query,
                "excludeKeyword": "",
                "sort": "SORT_SCORE",
                "order": "ORDER_DESC",
                "status": [],
                "sizeId": sizes,
                "categoryId": categories,
                "brandId": brands,
                "sellerId": [],
                "priceMin": price_min,
                "priceMax": price_max,
                "itemConditionId": item_conditions,
                "shippingPayerId": shipping_payer,
                "shippingFromArea": [],
                "shippingMethod": shipping_methods,
                "colorId": colors,
                "hasCoupon": False,
                "attributes": [],
                "itemTypes": [],
                "skuIds": [],
            },
            "defaultDatasets": [],
            "serviceFrom": "suruga",
        }



from mercapi.models.base import ResponseModel, Extractors, ResponseProperty


class Meta(ResponseModel):
    _required_properties = [
        ResponseProperty(
            "nextPageToken", "next_page_token", Extractors.get("nextPageToken")
        ),
        ResponseProperty(
            "previousPageToken", "prev_page_token", Extractors.get("previousPageToken")
        ),
        ResponseProperty("numFound", "num_found", Extractors.get_as("numFound", int)),
    ]
    _optional_properties = []

    def __init__(self, next_page_token: str, prev_page_token: str, num_found: int):
        super().__init__()
        self.next_page_token = next_page_token
        self.prev_page_token = prev_page_token
        self.num_found = num_found



import random
from time import time
from typing import Dict, Optional
from uuid import UUID

from ecdsa import SigningKey
from jose import jws
from jose.backends.ecdsa_backend import ECDSAECKey
from jose.constants import ALGORITHMS


def generate_dpop(
    url: str,
    method: str,
    key: SigningKey,
    extra_payload: Optional[Dict[str, str]] = None,
) -> str:
    payload = {
        "iat": int(time()),
        "jti": str(UUID(int=random.getrandbits(128))),
        "htu": url,
        "htm": method,
        **extra_payload,
    }

    ec_key = ECDSAECKey(key, ALGORITHMS.ES256)
    headers = {
        "typ": "dpop+jwt",
        "alg": "ES256",
        "jwk": {k: ec_key.to_dict()[k] for k in ["crv", "kty", "x", "y"]},
    }

    return jws.sign(payload, key, headers, ALGORITHMS.ES256)



import random
import uuid
from typing import Optional, List

import httpx
from ecdsa import SigningKey, NIST256p
from httpx import Request

from mercapi.models import SearchResults, Item, Profile, Items
from mercapi.models.base import ResponseModel
from mercapi.requests import SearchRequestData
from mercapi.util import jwt


class Mercapi:
    """Main class of the module containing all implemented
    wrappers of Mercari API.

    A random key-pair will be generated during the class instantiation.
    It is used for signing all HTTP requests sent in the course of methods execution.

    **Avoid instantiating this class more than once in a single runtime.**
    """

    _user_agent = (
        "Mozilla/5.0 (X11; Linux x86_64; rv:102.0) Gecko/20100101 Firefox/102.0"
    )
    _headers = {
        "User-Agent": _user_agent,
        "X-Platform": "web",
    }

    def __init__(self):
        self._uuid = str(uuid.UUID(int=random.getrandbits(128)))
        self._key = SigningKey.generate(NIST256p)
        self._client = httpx.AsyncClient()
        ResponseModel.set_mercapi(self)

    def _sign_request(self, request: Request) -> Request:
        request.headers["DPoP"] = jwt.generate_dpop(
            str(request.url),
            request.method,
            self._key,
            {
                "uuid": self._uuid,
            },
        )
        return request

    async def search(
        self,
        query: str,
        categories: List[int] = [],
        brands: List[int] = [],
        sizes: List[int] = [],
        price_min: int = None,
        price_max: int = None,
        item_conditions: List[int] = [],
        shipping_payer: List[int] = [],
        colors: List[int] = [],
        shipping_methods: List[SearchRequestData.ShippingMethod] = [],
        status: List[SearchRequestData.Status] = [],
    ) -> SearchResults:
        """Perform basic search and return list of items and metadata.
        This method reflects the action of using search bar at the top of the website.

        All parameters except for `query` must be provided as lists of ints
        referencing facets IDs supplied by Mercari API. Refer to files in
        `docs/facets` directory enumerating available facets and their identifiers.

        These files can be updated at any time by running `utils/fetch_facets.py`.

        :param query: string results should match (e.g. what you type in top search bar)
        :param categories: filter results by categories (カテゴリー)
        :param brands: filter results by brands (ブランド)
        :param sizes: filter results by sizes (サイズ)
        :param price_min: set results minimum price (価格, first textbox)
        :param price_max: set results maximum price (価格, second textbox)
        :param item_conditions: filter results by conditions  (商品の状態)
        :param shipping_payer: filter results by party responsible for reimbursing shipping costs (配送料の負担)
        :param colors: filter results by colors (色)
        :param shipping_methods: filter results by available shipping methods (発送オプション)
        :param status: filter results by listing statuses (販売状況)
        :return: List of search results (items) and metadata (e.g. total count)
        """
        request = SearchRequestData(
            SearchRequestData.SearchConditions(
                query,
                categories,
                brands,
                sizes,
                price_min,
                price_max,
                item_conditions,
                shipping_payer,
                colors,
                shipping_methods,
                status,
            ),
        )
        res = await self._client.send(self._search(request))
        body = res.json()
        return SearchResults.from_dict(body)

    def _search(self, search_request_data: SearchRequestData) -> Request:
        req = Request(
            "POST",
            "https://api.mercari.jp/v2/entities:search",
            json=search_request_data.data,
            headers=self._headers,
        )
        return self._sign_request(req)

    async def item(self, id_: str) -> Optional[Item]:
        """Fetch details of a single listing (item).
        This method reflects the action of loading single item view.

        :param id_: id of a listing (item)
        :return: all available listing (item) properties
        """
        res = await self._client.send(self._item(id_))
        if res.status_code == 404:
            return None

        body = res.json()
        return Item.from_dict(body["data"])

    def _item(self, id_: str) -> Request:
        req = Request(
            "GET",
            "https://api.mercari.jp/items/get",
            params={"id": id_},
            headers=self._headers,
        )
        return self._sign_request(req)

    async def profile(self, id_: str) -> Optional[Profile]:
        """Fetch details of a single seller.
        This method reflects the action of loading single seller profile view.

        :param id_: id of a seller (profile)
        :return: all available seller (profile) properties
        """
        res = await self._client.send(self._profile(id_))
        if res.status_code == 404:
            return None

        body = res.json()
        return Profile.from_dict(body["data"])

    def _profile(self, id_: str) -> Request:
        req = Request(
            "GET",
            "https://api.mercari.jp/users/get_profile",
            params={"user_id": id_, "_user_format": "profile"},
            headers=self._headers,
        )
        return self._sign_request(req)

    async def items(self, profile_id: str) -> Optional[Items]:
        """Fetch all items sold by specified seller.
        This method reflects the action of loading single seller profile view.

        :param profile_id: ID of a seller
        :return: list of items sold by specified seller
        """
        res = await self._client.send(self._items(profile_id))
        if res.status_code == 404:
            return None

        body = res.json()
        return Items.from_dict(body)

    def _items(self, profile_id: str) -> Request:
        req = Request(
            "GET",
            "https://api.mercari.jp/items/get_items",
            params={
                "seller_id": profile_id,
                "limit": 30,
                "status": "on_sale,trading,sold_out",
            },
            headers=self._headers,
        )
        return self._sign_request(req)



from typing import List, TypeVar

from mercapi.models.base import Extractors, ResponseModel, ResponseProperty

IC = TypeVar("IC", bound="ItemCategory")


class ItemCategory(ResponseModel):
    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty("name", "name", Extractors.get("name")),
    ]
    _optional_properties = [
        ResponseProperty(
            "display_order", "display_order", Extractors.get("display_order")
        ),
        ResponseProperty("tab_order", "tab_order", Extractors.get("tab_order")),
        ResponseProperty(
            "parent_category_id",
            "parent_category_id",
            Extractors.get("parent_category_id"),
        ),
        ResponseProperty(
            "parent_category_name",
            "parent_category_name",
            Extractors.get("parent_category_name"),
        ),
        ResponseProperty(
            "root_category_id", "root_category_id", Extractors.get("root_category_id")
        ),
        ResponseProperty(
            "root_category_name",
            "root_category_name",
            Extractors.get("root_category_name"),
        ),
        ResponseProperty(
            "size_group_id", "size_group_id", Extractors.get("size_group_id")
        ),
        ResponseProperty(
            "brand_group_id", "brand_group_id", Extractors.get("brand_group_id")
        ),
    ]

    def __init__(
        self,
        id_: int,
        name: str,
        tab_order: int,
        display_order: int,
        parent_category_id: int,
        parent_category_name: str,
        root_category_id: int,
        root_category_name: str,
        size_group_id: int,
        brand_group_id: int,
        children: List["ItemCategory"],
    ):
        super().__init__()
        self.id_ = id_
        self.name = name
        self.tab_order = tab_order
        self.display_order = display_order
        self.parent_category_id = parent_category_id
        self.parent_category_name = parent_category_name
        self.root_category_id = root_category_id
        self.root_category_name = root_category_name
        self.size_group_id = size_group_id
        self.brand_group_id = brand_group_id
        self.children = children

    @classmethod
    def from_dict(cls, d: dict) -> IC:
        # Dirty workaround for recursively parsing 'child' property which is a list
        # of ItemCategory objects. As a class cannot be self-referenced inside default properties,
        # like _required_properties and _optional_properties lists, this element is appended here
        # making use of 'cls' parameter.
        #
        # Alternative way of passing type to Extractors.get_list_of_model using module and class name
        # is not possible here due to cyclic imports hierarchy.
        if not any(raw_name == "child" for raw_name, _, _ in cls._optional_properties):
            cls._optional_properties.append(
                ResponseProperty(
                    "child", "children", Extractors.get_list_of_model("child", cls)
                )
            )
        return super().from_dict(d)



from datetime import datetime
from typing import List

from mercapi.models.base import Extractors, ResponseModel, ResponseProperty
from mercapi.models.common import ItemCategory
from mercapi.models.item.data import ShippingFromArea


class SellerItem(ResponseModel):
    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty(
            "seller",
            "seller_id",
            Extractors.get_with(
                "seller_id", lambda x: str(x["id"]) if "id" in x else None
            ),
        ),
        ResponseProperty("status", "status", Extractors.get("status")),
        ResponseProperty("name", "name", Extractors.get("name")),
        ResponseProperty("price", "price", Extractors.get("price")),
    ]
    _optional_properties = [
        ResponseProperty("thumbnails", "thumbnails", Extractors.get("thumbnails")),
        ResponseProperty(
            "root_category_id", "root_category_id", Extractors.get("root_category_id")
        ),
        ResponseProperty("num_likes", "num_likes", Extractors.get("num_likes")),
        ResponseProperty(
            "num_comments", "num_comments", Extractors.get("num_comments")
        ),
        ResponseProperty("created", "created", Extractors.get_datetime("created")),
        ResponseProperty("updated", "updated", Extractors.get_datetime("updated")),
        ResponseProperty(
            "item_category",
            "item_category",
            Extractors.get_as_model("item_category", ItemCategory),
        ),
        ResponseProperty(
            "shipping_from_area",
            "shipping_from_area",
            Extractors.get_as_model("shipping_from_area", ShippingFromArea),
        ),
    ]

    def __init__(
        self,
        id_: str,
        seller_id: str,
        status: str,
        name: str,
        price: int,
        thumbnails: List[str],
        root_category_id: int,
        num_likes: int,
        num_comments: int,
        created: datetime,
        updated: datetime,
        item_category: ItemCategory,
        shipping_from_area: ShippingFromArea,
    ):
        self.id_ = id_
        self.seller_id = seller_id
        self.status = status
        self.name = name
        self.price = price
        self.thumbnails = thumbnails
        self.root_category_id = root_category_id
        self.num_likes = num_likes
        self.num_comments = num_comments
        self.created = created
        self.updated = updated
        self.item_category = item_category
        self.shipping_from_area = shipping_from_area


class Items(ResponseModel):
    _required_properties = [
        ResponseProperty(
            "data", "items", Extractors.get_list_of_model("data", SellerItem)
        ),
    ]
    _optional_properties = []

    def __init__(self, items: List[SellerItem]):
        super().__init__()
        self.items = items



from datetime import datetime

from mercapi.models.base import Extractors, ResponseModel, ResponseProperty


class Seller(ResponseModel):
    class Ratings(ResponseModel):
        _required_properties = [
            ResponseProperty("good", "good", Extractors.get("good")),
            ResponseProperty("normal", "normal", Extractors.get("normal")),
            ResponseProperty("bad", "bad", Extractors.get("bad")),
        ]
        _optional_properties = []

        def __init__(self, good: int, normal: int, bad: int):
            super().__init__()
            self.good = good
            self.normal = normal
            self.bad = bad

    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty("name", "name", Extractors.get("name")),
    ]
    _optional_properties = [
        ResponseProperty("photo_url", "photo", Extractors.get("photo_url")),
        ResponseProperty(
            "photo_thumbnail_url",
            "photo_thumbnail",
            Extractors.get("photo_thumbnail_url"),
        ),
        ResponseProperty(
            "register_sms_confirmation",
            "register_sms_confirmation",
            Extractors.get("register_sms_confirmation"),
        ),
        ResponseProperty(
            "register_sms_confirmation_at",
            "register_sms_confirmation_at",
            Extractors.get_with(
                "register_sms_confirmation_at",
                lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S"),
            ),
        ),
        ResponseProperty("created", "created", Extractors.get_datetime("created")),
        ResponseProperty(
            "num_sell_items", "num_sell_items", Extractors.get("num_sell_items")
        ),
        ResponseProperty(
            "ratings",
            "ratings",
            Extractors.get_with("ratings", lambda x: Seller.Ratings.from_dict(x)),
        ),
        ResponseProperty("num_ratings", "num_ratings", Extractors.get("num_ratings")),
        ResponseProperty("score", "score", Extractors.get("score")),
        ResponseProperty("is_official", "is_official", Extractors.get("is_official")),
        ResponseProperty(
            "quick_shipper", "quick_shipper", Extractors.get("quick_shipper")
        ),
        ResponseProperty(
            "star_rating_score",
            "star_rating_score",
            Extractors.get("star_rating_score"),
        ),
    ]

    def __init__(
        self,
        id_: int,
        name: str,
        photo: str,
        photo_thumbnail: str,
        register_sms_confirmation: str,
        register_sms_confirmation_at: datetime,
        created: datetime,
        num_sell_items: int,
        ratings: Ratings,
        num_ratings: int,
        score: int,
        is_official: bool,
        quick_shipper: bool,
        star_rating_score: int,
    ):
        super().__init__()
        self.id_ = id_
        self.name = name
        self.photo = photo
        self.photo_thumbnail = photo_thumbnail
        self.register_sms_confirmation = register_sms_confirmation
        self.register_sms_confirmation_at = register_sms_confirmation_at
        self.created = created
        self.num_sell_items = num_sell_items
        self.ratings = ratings
        self.num_ratings = num_ratings
        self.score = score
        self.is_official = is_official
        self.quick_shipper = quick_shipper
        self.star_rating_score = star_rating_score


class ItemCondition(ResponseModel):
    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty("name", "name", Extractors.get("name")),
    ]
    _optional_properties = []

    def __init__(self, id_: int, name: str):
        super().__init__()
        self.id_ = id_
        self.name = name


class Color(ResponseModel):
    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty("name", "name", Extractors.get("name")),
        ResponseProperty(
            "rgb", "rgb", Extractors.get_with("rgb", lambda x: int(x[1:], 16))
        ),
    ]
    _optional_properties = []

    def __init__(self, id_: int, name: str, rgb: int):
        super().__init__()
        self.id_ = id_
        self.name = name
        self.rgb = rgb

    @property
    def rgb_code(self) -> str:
        return hex(self.rgb)


class ShippingPayer(ResponseModel):
    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty("name", "name", Extractors.get("name")),
    ]
    _optional_properties = [
        ResponseProperty("code", "code", Extractors.get("code")),
    ]

    def __init__(self, id_: int, name: str, code: str):
        super().__init__()
        self.id_ = id_
        self.name = name
        self.code = code


class ShippingMethod(ResponseModel):
    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty("name", "name", Extractors.get("name")),
    ]
    _optional_properties = [
        ResponseProperty(
            "is_deprecated", "is_deprecated", Extractors.get("is_deprecated")
        ),
    ]

    def __init__(self, id_: int, name: str, is_deprecated: str):
        super().__init__()
        self.id_ = id_
        self.name = name
        self.is_deprecated = is_deprecated


class ShippingFromArea(ResponseModel):
    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty("name", "name", Extractors.get("name")),
    ]
    _optional_properties = []

    def __init__(self, id_: int, name: str):
        super().__init__()
        self.id_ = id_
        self.name = name


class ShippingDuration(ResponseModel):
    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty("name", "name", Extractors.get("name")),
        ResponseProperty("min_days", "min_days", Extractors.get("min_days")),
        ResponseProperty("max_days", "max_days", Extractors.get("max_days")),
    ]
    _optional_properties = []

    def __init__(self, id_: int, name: str, min_days: int, max_days: int):
        super().__init__()
        self.id_ = id_
        self.name = name
        self.min_days = min_days
        self.max_days = max_days


class ShippingClass(ResponseModel):
    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty("fee", "fee", Extractors.get("fee")),
        ResponseProperty("icon_id", "icon_id", Extractors.get("icon_id")),
        ResponseProperty("pickup_fee", "pickup_fee", Extractors.get("pickup_fee")),
        ResponseProperty(
            "shipping_fee", "shipping_fee", Extractors.get("shipping_fee")
        ),
        ResponseProperty("total_fee", "total_fee", Extractors.get("total_fee")),
        ResponseProperty("is_pickup", "is_pickup", Extractors.get("is_pickup")),
    ]
    _optional_properties = []

    def __init__(
        self,
        id_: int,
        fee: int,
        icon_id: int,
        pickup_fee: int,
        shipping_fee: int,
        total_fee: int,
        is_pickup: bool,
    ):
        super().__init__()
        self.id_ = id_
        self.fee = fee
        self.icon_id = icon_id
        self.pickup_fee = pickup_fee
        self.shipping_fee = shipping_fee
        self.total_fee = total_fee
        self.is_pickup = is_pickup


class Comment(ResponseModel):
    class User(ResponseModel):
        _required_properties = [
            ResponseProperty("id", "id_", Extractors.get("id")),
            ResponseProperty("name", "name", Extractors.get("name")),
        ]
        _optional_properties = [
            ResponseProperty("photo_url", "photo", Extractors.get("photo_url")),
            ResponseProperty(
                "photo_thumbnail_url",
                "photo_thumbnail",
                Extractors.get("photo_thumbnail_url"),
            ),
        ]

        def __init__(self, id_: int, name: str, photo: str, photo_thumbnail: str):
            super().__init__()
            self.id_ = id_
            self.name = name
            self.photo = photo
            self.photo_thumbnail = photo_thumbnail

    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty("message", "message", Extractors.get("message")),
    ]
    _optional_properties = [
        ResponseProperty("user", "user", Extractors.get_as_model("user", User)),
        ResponseProperty("created", "created", Extractors.get_datetime("created")),
    ]



from datetime import datetime
from typing import List, TYPE_CHECKING

if TYPE_CHECKING:
    from mercapi.models import Item, Profile
from mercapi.models.base import Extractors, ResponseModel, ResponseProperty


class SearchResultItem(ResponseModel):
    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty("name", "name", Extractors.get("name")),
        ResponseProperty("price", "price", Extractors.get_as("price", int)),
    ]
    _optional_properties = [
        ResponseProperty("sellerId", "seller_id", Extractors.get("sellerId")),
        ResponseProperty("status", "status", Extractors.get("status")),
        ResponseProperty("created", "created", Extractors.get_datetime("created")),
        ResponseProperty("updated", "updated", Extractors.get_datetime("updated")),
        ResponseProperty("buyerId", "buyer_id", Extractors.get("buyerId")),
        ResponseProperty("thumbnails", "thumbnails", Extractors.get("thumbnails")),
        ResponseProperty("itemType", "item_type", Extractors.get("itemType")),
        ResponseProperty(
            "itemConditionId",
            "item_condition_id",
            Extractors.get_as("itemConditionId", int),
        ),
        ResponseProperty(
            "shippingPayerId",
            "shipping_payer_id",
            Extractors.get_as("shippingPayerId", int),
        ),
    ]

    def __init__(
        self,
        id_: str,
        name: str,
        price: int,
        seller_id: str,
        status: str,
        created: datetime,
        updated: datetime,
        buyer_id: str,
        thumbnails: List[str],
        item_type: str,
        item_condition_id: int,
        shipping_payer_id: int,
    ):
        super().__init__()
        self.id_ = id_
        self.name = name
        self.price = price
        self.seller_id = seller_id
        self.status = status
        self.created = created
        self.updated = updated
        self.buyer_id = buyer_id
        self.thumbnails = thumbnails
        self.item_type = item_type
        self.item_condition_id = item_condition_id
        self.shipping_payer_id = shipping_payer_id

    async def full_item(self) -> "Item":
        return await self._mercapi.item(self.id_)

    async def seller(self) -> "Profile":
        return await self._mercapi.profile(self.seller_id)



import logging
from datetime import datetime
from typing import (
    NamedTuple,
    Callable,
    List,
    TypeVar,
    Any,
    Type,
    TYPE_CHECKING,
    Optional,
)

if TYPE_CHECKING:
    from mercapi import Mercapi
from mercapi.util.errors import ParseAPIResponseError


RM = TypeVar("RM", bound="ResponseModel")
T = TypeVar("T")
ExtractorDef = Callable[[dict], Optional[T]]


class ResponseProperty(NamedTuple):
    raw_property_name: str
    model_property_name: str
    extractor: ExtractorDef


class ResponseModel:

    _mercapi: "Mercapi"

    _required_properties: List[ResponseProperty]
    _optional_properties: List[ResponseProperty]

    def __init__(self, *args, **kwargs):
        pass

    def keys(self):
        return [
            p.model_property_name
            for p in self._required_properties + self._optional_properties
        ]

    def __getitem__(self, key):
        if key not in self.keys():
            raise IndexError(
                f"{key} is not a valid member of {self.__class__.__name__}"
            )
        res = getattr(self, key)
        if not issubclass(type(res), ResponseModel):
            return res
        return dict(res)

    @classmethod
    def from_dict(cls, d: dict) -> RM:
        if cls == ResponseModel:
            raise TypeError(
                "from_dict() class method is supposed to be called on ResponseModel subclasses providing "
                "lists of properties"
            )

        model_properties = {}

        for prop in cls._required_properties:
            try:
                raw_property = prop.extractor(d)
                model_properties[prop.model_property_name] = raw_property
            except Exception as e:
                raise ParseAPIResponseError(
                    f"Failed to retrieve required {cls.__name__} property {prop.raw_property_name} from the response"
                ) from e

        for prop in cls._optional_properties:
            raw_property = None
            try:
                raw_property = prop.extractor(d)
            except Exception as e:
                cls._report_incorrect_optional(prop.raw_property_name, d, e)
            model_properties[prop.model_property_name] = raw_property

        return cls(**model_properties)

    @classmethod
    def set_mercapi(cls, m: "Mercapi") -> None:
        cls._mercapi = m

    @classmethod
    def _report_incorrect_optional(
        cls, prop: str, response: dict, exc: Exception
    ) -> None:
        logging.warning(
            f"Encountered optional response property {prop} that could not be parsed correctly."
        )
        logging.debug(f"Response body: {response}\nError: {exc}")


class Extractors:
    """
    Collection of HOFs for parsing API responses in the most common ways.

    Each extractor function MUST handle lack of requested key
    in the response object (dict) and return None in such cases.
    """

    @staticmethod
    def get(key: str) -> ExtractorDef[Any]:
        return lambda x: x.get(key)

    S = TypeVar("S", int, float, str)

    @staticmethod
    def get_as(key: str, type_: Type[S]) -> ExtractorDef[S]:
        return lambda x: type_(x[key]) if key in x else None

    M = TypeVar("M", bound=ResponseModel)

    @staticmethod
    def get_as_model(key: str, model: Type[M]) -> ExtractorDef[M]:
        if type(model) == str:
            model = Extractors.__import_class(model)
        return lambda x: model.from_dict(x[key]) if key in x else None

    @staticmethod
    def get_with(key: str, mapper: Callable[[S], T]) -> ExtractorDef[T]:
        return lambda x: mapper(x[key]) if key in x else None

    @staticmethod
    def get_list_with(key: str, mapper: Callable[[Any], T]) -> ExtractorDef[List[T]]:
        return lambda x: [mapper(i) for i in x[key]] if key in x else None

    @staticmethod
    def get_list_of_model(key: str, model: Type[M]) -> ExtractorDef[List[M]]:
        if type(model) == str:
            model = Extractors.__import_class(model)
        return lambda x: [model.from_dict(i) for i in x[key]] if key in x else None

    @staticmethod
    def get_datetime(key: str) -> ExtractorDef[datetime]:
        return Extractors.get_with(key, lambda x: datetime.fromtimestamp(float(x)))

    @staticmethod
    def __import_class(model: str) -> Type[ResponseModel]:
        import importlib

        module_name, class_name = model.rsplit(".", 1)
        return getattr(importlib.import_module(module_name), class_name)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mercapi',
 'mercapi.models',
 'mercapi.models.item',
 'mercapi.models.profile',
 'mercapi.models.search',
 'mercapi.requests',
 'mercapi.util']

package_data = \
{'': ['*']}

install_requires = \
['ecdsa>=0.18.0,<0.19.0',
 'httpx>=0.23.0,<0.24.0',
 'python-jose[cryptography]>=3.3.0,<4.0.0']

setup_kwargs = {
    'name': 'mercapi',
    'version': '0.2.1',
    'description': 'Python API for querying and browsing mercari.jp',
    'long_description': "# mercapi\n\n![PyPI](https://img.shields.io/pypi/v/mercapi)\n[![Tests](https://github.com/take-kun/mercapi/actions/workflows/check.yaml/badge.svg?branch=main)](https://github.com/take-kun/mercapi/actions/workflows/check.yaml)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/mercapi)\n\n[API Documentation](https://take-kun.github.io/mercapi/)\n\n## What is Mercapi?\n\nMercapi is a Python wrapper for *mercari.jp* API.\nIt's capable of producing HTTP requests implementing security mechanisms employed in native *mercari.jp* web app.\nRequests and responses are mapped to custom classes with type-hinting and documentation.\n\n## Quickstart\n\nFirst, install the `mercapi` package using the package manager of your choice.\n\nAs an example, we want to run the search query `sharpnel`.\n\n```python\nfrom mercapi import Mercapi\n\n\nm = Mercapi()\nresults = await m.search('sharpnel')\n\nprint(f'Found {results.meta.num_found} results')\nfor item in results.items:\n    print(f'Name: {item.name}\\\\nPrice: {item.price}\\\\n')\n\n```\n\nWe can use a single result object to retrieve full details of the listing.\n```python\nitem = results.items[0]\nfull_item = await item.full_item()\n\nprint(full_item.description)\n```\n\nOr get it directly using an ID.\n```python\nitem = await m.item('m90925725213')\n\nprint(item.description)\n```\n\nRefer to `mercapi.mercapi.Mercapi` documentation for all implemented features.\n\n*Examples above are not executable. If you want to try them out, run `python example.py`.*",
    'author': 'take-kun',
    'author_email': '109226194+take-kun@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/take-kun/mercapi/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import List

from mercapi.models.base import Extractors, ResponseModel, ResponseProperty
from mercapi.models.search import SearchResultItem, Meta


class SearchResults(ResponseModel):
    _required_properties = [
        ResponseProperty("meta", "meta", Extractors.get_as_model("meta", Meta)),
        ResponseProperty(
            "items",
            "items",
            Extractors.get_list_with("items", lambda i: SearchResultItem.from_dict(i)),
        ),
    ]
    _optional_properties = []

    def __init__(self, meta: Meta, items: List[SearchResultItem]):
        super().__init__()
        self.meta = meta
        self.items = items
        self._request = None



from datetime import datetime
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from mercapi.models import Items
from mercapi.models.base import Extractors, ResponseModel, ResponseProperty


class Profile(ResponseModel):
    class Ratings(ResponseModel):
        _required_properties = [
            ResponseProperty("good", "good", Extractors.get("good")),
            ResponseProperty("normal", "normal", Extractors.get("normal")),
            ResponseProperty("bad", "bad", Extractors.get("bad")),
        ]
        _optional_properties = []

        def __init__(self, good: int, normal: int, bad: int):
            super().__init__()
            self.good = good
            self.normal = normal
            self.bad = bad

    class PolarizedRatings(ResponseModel):
        _required_properties = [
            ResponseProperty("good", "good", Extractors.get("good")),
            ResponseProperty("bad", "bad", Extractors.get("bad")),
        ]
        _optional_properties = []

        def __init__(self, good: int, bad: int):
            super().__init__()
            self.good = good
            self.bad = bad

    _required_properties = [
        ResponseProperty("id", "id_", Extractors.get("id")),
        ResponseProperty("name", "name", Extractors.get("name")),
    ]

    _optional_properties = [
        ResponseProperty("photo_url", "photo_url", Extractors.get("photo_url")),
        ResponseProperty(
            "photo_thumbnail_url",
            "photo_thumbnail_url",
            Extractors.get("photo_thumbnail_url"),
        ),
        ResponseProperty(
            "register_sms_confirmation",
            "register_sms_confirmation",
            Extractors.get("register_sms_confirmation"),
        ),
        ResponseProperty(
            "ratings",
            "ratings",
            Extractors.get_with("ratings", lambda x: Profile.Ratings.from_dict(x)),
        ),
        ResponseProperty(
            "polarized_ratings",
            "polarized_ratings",
            Extractors.get_with(
                "polarized_ratings", lambda x: Profile.PolarizedRatings.from_dict(x)
            ),
        ),
        ResponseProperty("num_ratings", "num_ratings", Extractors.get("num_ratings")),
        ResponseProperty(
            "star_rating_score",
            "star_rating_score",
            Extractors.get("star_rating_score"),
        ),
        ResponseProperty(
            "is_followable", "is_followable", Extractors.get("is_followable")
        ),
        ResponseProperty("is_blocked", "is_blocked", Extractors.get("is_blocked")),
        ResponseProperty(
            "following_count", "following_count", Extractors.get("following_count")
        ),
        ResponseProperty(
            "follower_count", "follower_count", Extractors.get("follower_count")
        ),
        ResponseProperty("score", "score", Extractors.get("score")),
        ResponseProperty("created", "created", Extractors.get_datetime("created")),
        ResponseProperty("proper", "proper", Extractors.get("proper")),
        ResponseProperty(
            "introduction", "introduction", Extractors.get("introduction")
        ),
        ResponseProperty("is_official", "is_official", Extractors.get("is_official")),
        ResponseProperty(
            "num_sell_items", "num_sell_items", Extractors.get("num_sell_items")
        ),
        ResponseProperty("num_ticket", "num_ticket", Extractors.get("num_ticket")),
        ResponseProperty(
            "bounce_mail_flag", "bounce_mail_flag", Extractors.get("bounce_mail_flag")
        ),
        # useless without authorization context
        # ('is_following', 'is_following', Extractors.get('is_following'))
        ResponseProperty(
            "current_point", "current_point", Extractors.get("current_point")
        ),
        ResponseProperty(
            "current_sales", "current_sales", Extractors.get("current_sales")
        ),
        ResponseProperty(
            "is_organizational_user",
            "is_organizational_user",
            Extractors.get("is_organizational_user"),
        ),
    ]

    def __init__(
        self,
        id_: str,
        name: str,
        photo_url: str,
        photo_thumbnail_url: str,
        register_sms_confirmation: str,
        ratings: Ratings,
        polarized_ratings: PolarizedRatings,
        num_ratings: int,
        star_rating_score: int,
        is_followable: bool,
        is_blocked: bool,
        following_count: int,
        follower_count: int,
        score: int,
        created: datetime,
        proper: bool,
        introduction: str,
        is_official: bool,
        num_sell_items: int,
        num_ticket: int,
        bounce_mail_flag: str,
        current_point: int,
        current_sales: int,
        is_organizational_user: bool,
    ):
        super().__init__()
        self.id_ = id_
        self.name = name
        self.photo_url = photo_url
        self.photo_thumbnail_url = photo_thumbnail_url
        self.register_sms_confirmation = register_sms_confirmation
        self.ratings = ratings
        self.polarized_ratings = polarized_ratings
        self.num_ratings = num_ratings
        self.star_rating_score = star_rating_score
        self.is_followable = is_followable
        self.is_blocked = is_blocked
        self.following_count = following_count
        self.follower_count = follower_count
        self.score = score
        self.created = created
        self.proper = proper
        self.introduction = introduction
        self.is_official = is_official
        self.num_sell_items = num_sell_items
        self.num_ticket = num_ticket
        self.bounce_mail_flag = bounce_mail_flag
        self.current_point = current_point
        self.current_sales = current_sales
        self.is_organizational_user = is_organizational_user

    async def items(self) -> "Items":
        return await self._mercapi.items(self.id_)
